-- -- COPYRIGHT NOTICE -- --
-- This work is covered by the LGPL License. You are
-- able to modify, distribute and sublicense this
-- work as long as you include the original or show
-- clearly how to obtain it.

-- NOTES:
-- - Filename:   Gui.lua
-- - Written By: Trilesch and Digpoe
-- - Purpose:    Defines the CombatStatus Gui instance.

-- Lots of metatable tricks to make my life easier.
-- Feel free to use this as a library, I'll probably be
-- doing something like that myself once the plugin is
-- completed.

Gui = { }

local function GetChildByName(gui, child)
	assert(rawget(gui, "children"), "attempt to set parent to unwrapped object")
	for i, v in pairs(rawget(gui, "children")) do
		if v.Name == child then
			return v
		end
	end
end

-- Local Gui object
local iGui = { }
iGui.__index = function(self, value)
	assert(value ~= "children", "attempt to index 'children' (a write-protected value)")
	if value == "__implementation" then return rawget(self, "win")[value] end
	return rawget(self, "props")[value] or GetChildByName(self, value) or error("Attempt to index "..value.." (a nil value)")
end

--- Returns the children of the Gui
-- @function GetChildren
-- @return Table of children
function iGui:GetChildren()
	return rawget(Gui, "children")
end

iGui.__newindex = function(self, key, value)
	if key == "Position" then
		assert(type(value) == "table", "table expected, got "..type(value))
		rawget(self, "win"):SetPosition(value.X, value.Y)
	elseif key == "Size" then
		assert(type(value) == "table", "table expected, got "..type(value))
		rawget(self, "win"):SetSize(value.X, value.Y)
	elseif key == "Rect" then
		assert(type(value) == "table", "table expected, got "..type(value))
		rawget(self, "win"):SetPosition(value.X, value.Y)
		rawget(self, "win"):SetSize(value.Width, value.Height)
	elseif rawget(self, "win").SetText and key == "Text" then
		assert(type(value) == "string", "string expected, got "..type(value))
		rawget(self, "win"):SetText(value)
	elseif key == "Name" then
		assert(type(value) == "string", "string expected, got "..type(value))
	elseif key == "Visible" then
		assert(type(value) == "boolean", "boolean expected, got "..type(value))
		rawget(self, "win"):SetVisible(value)
	elseif key == "BackgroundImage" then
		assert(type(value) == "string", "string expected, got "..type(value))
		rawget(self, "win"):SetBackground(value)
	elseif key == "BackgroundColor3" then
		assert(type(value) == "table", "table expected, got "..type(value))
		rawget(self, "win"):SetBackColor(Turbine.UI.Color(rawget(self, "win"):GetBackColor().A, value.R, value.G, value.B))
	elseif key == "BackgroundTransparency" then
		assert(type(value) == "number", "number expected, got "..type(value))
		local color = rawget(self, "win"):GetBackColor()
		color.A = value
		rawget(self, "win"):SetBackColor(color)
	elseif key == "Parent" then
		assert(type(value) == "table", "table expected, got "..type(value))
		assert(rawget(value, "children"), "attempt to set parent to unwrapped object")
		local children = rawget(value, "children")
		children[#children+1] = self
		rawset(value, "children", children)
		rawget(self, "win"):SetParent(value)
	elseif key == "OnUpdate" then
		assert(type(value) == "function" or type(value) == "nil", "function expected, got "..type(value))
		if value then
			rawget(self, "win").Update = value
			rawget(self, "win"):SetWantsUpdates(true)
		else
			rawget(self, "win"):SetWantsUpdates(false)
			rawget(self, "win").Update = nil
		end
	elseif key == "ZIndex" then
		assert(type(value) == "number", "number expected, got "..type(value))
		rawget(self, "win"):SetZOrder(key)
	elseif rawget(self, "win").SetTextAlignment and key == "TextAlignment" then
		assert(type(value) == "number", "number expected, got "..type(value))
		rawget(self, "win"):SetTextAlignment(value)
	elseif rawget(self, "win").SetFont and key == "Font" then
		assert(type(value) == "number" or type(value) == "string", "Turbine.UI.Lotro.Font expected, got "..type(value))
		if type(value) == "number" then
			rawget(self, "win"):SetFont(value)
		else
			rawget(self, "win"):SetFont(Turbine.UI.Lotro.Font[value])
		end
	elseif rawget(self, "ClassName") == "Button" and key == "OnClick" then
		assert(type(value) == "function" or type(value) == "nil", "function expected, got "..type(value))
		rawget(self, "win").Click = value
	else
		error("attempt to set '"..key.."' (a write-protected value)")
	end
	self.props[key] = value
end

--- Creates a new Gui
-- @function new
-- @return the created Gui
function Gui.new(typ)
	assert(typ, "string expected")
	assert(Turbine.UI[typ], "unable to create an Instance of type \""..typ.."\"")
	
	local o = {win = Turbine.UI[typ](), children = { }, Name=typ, ClassName=typ, props={ }}
	
	if o.win.SetFont then
		o.win:SetFont(Turbine.UI.Lotro.Font.TrajanPro14)
	end
	
	return setmetatable(o, iGui)
end